"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const joycon_1 = __importDefault(require("joycon"));
const joycon_yaml_loader_1 = __importDefault(require("joycon-yaml-loader"));
const nvl_1 = __importDefault(require("nvl"));
const os_1 = require("os");
const path_1 = require("path");
const NAME = 'vue-pack';
/**
 * Load configuration from config file.
 * (If `privilegeConfig` is given, it'll override loaded config.)
 *
 * @export
 * @param {(Partial<Configuration> | Configuration)} [privilegeConfig] High priority config to override config from config file
 * @param {string} [configPath] Path to config file
 * @returns {Promise<Configuration>}
 */
function loadConfig(privilegeConfig, configPath) {
    return __awaiter(this, void 0, void 0, function* () {
        const joycon = new joycon_1.default({
            files: [
                `${NAME}.config.js`,
                `${NAME}.config.json`,
                `${NAME}.config.yaml`,
                `${NAME}.config.yml`,
                `${NAME}.config.ts`
            ],
            stopDir: os_1.homedir()
        });
        // YAML loader
        joycon.addLoader(joycon_yaml_loader_1.default);
        // TypeScript loader
        try {
            const JoyConTSLoader = require('joycon-ts-loader');
            joycon.addLoader(JoyConTSLoader);
        }
        catch (_) { }
        let config;
        if (configPath) {
            const configDir = path_1.resolve(path_1.dirname(configPath));
            const configFile = path_1.basename(configPath);
            const { data } = yield joycon.load([configFile], configDir);
            config = nvl_1.default(data, {});
        }
        else {
            const { data } = yield joycon.load();
            config = nvl_1.default(data, {});
        }
        config = privilegeConfig ? Object.assign({}, config, privilegeConfig) : config;
        return config;
    });
}
exports.default = loadConfig;
